# This code is part of the 'enjuewemela' game
# License: GPLv3
# Main author: Facundo Batista
# Code, bug tracker, etc:
#   https://launchpad.net/enjuewemela/
#
"""Menus of the game."""

from __future__ import division

from cocos.director import director
from cocos import menu, scene, layer
import pyglet

from sound import sounds, music_player
import highscores

AUDIO_VOLS = 'Mute 10 20 30 40 50 60 70 80 90 100'.split()


def set_options(config):
    """Configure the game according to saved config."""
    director.window.set_fullscreen(config['fullscreen'])
    sounds.volume = AUDIO_VOLS.index(config['sound_volume']) / 10
    music_player.volume = AUDIO_VOLS.index(config['music_volume']) / 10


class PlayWrapper(object):
    """A wrapper to be called by the menu."""
    def __init__(self, sound_name):
        self.sound_name = sound_name

    def play(self):
        """Plays the sound."""
        sounds.play(self.sound_name)


class OptionsMenu(menu.Menu):
    """The Options menu."""

    def __init__(self, game_manager):
        menu.Menu.__init__(self, _("Options"))
        self.config = game_manager.config
        self.select_sound = PlayWrapper('tic')

        # defaults
        d_uname = self.config['username']
        d_fscreen = director.window.fullscreen
        d_sound = AUDIO_VOLS.index(self.config['sound_volume'])
        d_music = AUDIO_VOLS.index(self.config['music_volume'])
        self.items = [
            (menu.EntryMenuItem(_('User name: '), self.on_username, d_uname)),
            (menu.ToggleMenuItem(_('Full screen: '),
                                 self.on_fullscreen, d_fscreen)),
            (menu.MultipleMenuItem(_('Sound volume: '), self.on_soundvol,
                                   AUDIO_VOLS, d_sound)),
            (menu.MultipleMenuItem(_('Music volume: '), self.on_musicvol,
                                   AUDIO_VOLS, d_music)),
            (menu.MenuItem(_('Back'), self.on_quit)),
        ]

        # then add the items
        self.create_menu(self.items, menu.shake(), menu.shake_back())

    def on_username(self, to_value):
        """Set the user name."""
        self.config['username'] = to_value

    def on_musicvol(self, to_value):
        """Set the music volume."""
        self.config['music_volume'] = AUDIO_VOLS[to_value]
        music_player.volume = to_value / 10

    def on_soundvol(self, to_value):
        """Set the sound volume."""
        self.config['sound_volume'] = AUDIO_VOLS[to_value]
        sounds.volume = to_value / 10
        sounds.play('tic')

    def on_fullscreen(self, to_value):
        """Toggle full screen."""
        director.window.set_fullscreen(to_value)
        self.config['fullscreen'] = bool(to_value)

    def on_quit(self):
        """Go back to the menu."""
        director.pop()

    def keep_playing(self):
        """Go back to the game!"""
        director.pop()


class CreditLayer(layer.Layer):
    """Background layer with the credits."""

    def __init__(self):
        super(CreditLayer, self).__init__()
        self.img = pyglet.resource.image('credits.png')

    def draw(self):
        """Called when the layer is drawn."""
        self.img.blit(0, 0)


def credits():
    """Show the credits."""
    sc = scene.Scene(CreditLayer())
    return sc


class PlayMenu(menu.Menu):
    """The Play menu."""

    def __init__(self, game_manager):
        menu.Menu.__init__(self, _("Play!"))

        self.gm = game_manager
        self.select_sound = PlayWrapper('tic')

        # then add the items
        self.items = [
            (menu.MenuItem(_('Classic'), self.on_classic)),
            (menu.MenuItem(_('Rush!'), self.on_rush)),
            (menu.MenuItem(_('Crazy'), self.on_crazy)),
            (menu.MenuItem(_('Back'), self.on_quit)),
        ]

        self.create_menu(self.items, menu.shake(), menu.shake_back())

    def on_quit(self):
        """Go back to the main menu."""
        self.gm.main_menu()

    def on_classic(self):
        """Classic mode."""
        sc = self.gm.play_classic()
        director.replace(sc)

    def on_rush(self):
        """Rush mode."""
        sc = self.gm.play_rush()
        director.replace(sc)

    def on_crazy(self):
        """Crazy mode."""
        sc = self.gm.play_crazy()
        director.replace(sc)


class SafeQuitMenu(menu.Menu):
    """Ask if sure to exit the game."""

    def __init__(self):
        menu.Menu.__init__(self, _("Have a game going on!"))

        self.select_sound = PlayWrapper('tic')

        # then add the items
        self.items = []

        self.items = [
            (menu.MenuItem(_('Keep playing!'), self.keep_playing)),
            (menu.MenuItem(_('Get me out of here'), self.really_exit)),
        ]

        self.create_menu(self.items, menu.shake(), menu.shake_back())

    def on_quit(self):
        """Handles the automatic escape, but keep playing."""
        self.keep_playing()

    def keep_playing(self):
        """Go back to the game!"""
        director.pop()

    def really_exit(self):
        """Quit the game."""
        pyglet.app.exit()


class MainMenu(menu.Menu):
    """Main menu for the game."""

    def __init__(self, game_manager, playing=False):
        menu.Menu.__init__(self, "Enjuewemela!")

        self.playing = playing
        self.gm = game_manager
        self.select_sound = PlayWrapper('tic')

        # then add the items
        self.items = []

        if self.playing:
            self.items.append(
                (menu.MenuItem(_('Continue previous game'), self.on_continue)),
        )

        self.items.extend([
            (menu.MenuItem(_('Play new game!'), self.on_play)),
            (menu.MenuItem(_('Options'), self.on_options)),
            (menu.MenuItem(_('High scores'), self.on_highscores)),
            (menu.MenuItem(_('Credits'), self.on_credits)),
            (menu.MenuItem(_('Quit'), self.on_quit)),
        ])

        self.create_menu(self.items, menu.shake(), menu.shake_back())

    def on_play(self):
        """Play the game!"""
        scn = scene.Scene()
        scn.add(PlayMenu(self.gm), z=1)
        director.replace(scn)

    def on_continue(self):
        """Continue the previous game."""
        self.gm.continue_game()

    def on_options(self):
        """Check options."""
        scn = scene.Scene()
        scn.add(OptionsMenu(self.gm), z=1)
        director.push(scn)

    def on_highscores(self):
        """Show the highscores."""
        scn = scene.Scene()
        scn.add(highscores.HighScores(), z=1)
        director.push(scn)

    def on_credits(self):
        """Who did the game."""
        scn = credits()
        director.push(scn)

    def on_quit(self):
        """Quit the game.

        If the user is in the middle of a game, ask for confirmation.
        """
        if self.playing:
            scn = scene.Scene()
            scn.add(SafeQuitMenu(), z=1)
            director.push(scn)
        else:
            pyglet.app.exit()
