# This code is part of the 'enjuewemela' game
# License: GPLv3
# Main author: Facundo Batista
# Code, bug tracker, etc:
#   https://launchpad.net/enjuewemela/
#
"""Menus of the game."""

import random


class FakeBoard(object):
    """Fake representation of the board."""

    def __init__(self, board):
        self.board = board
        self.height = len(board) - 1

    def get_piece(self, pos_x, pos_y):
        """Return the piece of the x, y position."""
        return self.board[self.height - pos_y][pos_x]

    def __len__(self):
        return len(self.board)

    def __str__(self):
        return "\n".join(str(x) for x in self.board)


def _cross(results):
    """Cross the results to see if they're composed."""
    lenres = len(results)
    for i in range(lenres):
        for j in range(i + 1, lenres):
            match = set(results[i]) & set(results[j])
            if match:
                return match.pop()


def _check_array(array):
    """Checks a single row or col to see if there's a match."""
    piece_which = None
    piece_counter = None
    for piece in array:
        if piece == piece_which:
            piece_counter += 1
            if piece_counter == 3:
                return True
        else:
            piece_which = piece
            piece_counter = 1
    return False


def future_guess(board):
    """Guess which piece the user could move to have a match."""
    lenboard = len(board)
    rangesize = range(lenboard)

    # horizontal
    for y in rangesize:
        row = [board.get_piece(x, y) for x in rangesize]

        for x in rangesize:
            prv = row[x]
            if y > 0:
                row[x] = board.get_piece(x, y - 1)
                if _check_array(row):
                    return x, y - 1

            if y < lenboard - 1:
                row[x] = board.get_piece(x, y + 1)
                if _check_array(row):
                    return x, y + 1
            row[x] = prv

            if x > 0:
                row[x - 1], row[x] = row[x], row[x - 1]
                if _check_array(row):
                    return x, y
                row[x], row[x - 1] = row[x - 1], row[x]

            if x < lenboard - 1:
                row[x + 1], row[x] = row[x], row[x + 1]
                if _check_array(row):
                    return x, y
                row[x], row[x + 1] = row[x + 1], row[x]

    # vertical
    for x in rangesize:
        col = [board.get_piece(x, y) for y in rangesize]

        for y in rangesize:
            if y > 0:
                col[y - 1], col[y] = col[y], col[y - 1]
                if _check_array(col):
                    return x, y
                col[y], col[y - 1] = col[y - 1], col[y]

            if y < lenboard - 1:
                col[y + 1], col[y] = col[y], col[y + 1]
                if _check_array(col):
                    return x, y
                col[y], col[y + 1] = col[y + 1], col[y]

            prv = col[y]
            if x > 0:
                col[y] = board.get_piece(x - 1, y)
                if _check_array(col):
                    return x - 1, y

            if x < lenboard - 1:
                col[y] = board.get_piece(x + 1, y)
                if _check_array(col):
                    return x + 1, y
            col[y] = prv


def detect(board):
    """Tell in which positions there're matches."""
    rangesize = range(len(board))

    all_found = {}

    # horizontal
    piece_which = None
    piece_counter = None
    found_mark = None

    for y in rangesize:
        for x in rangesize:
            this_piece = board.get_piece(x, y)
            if this_piece == piece_which:
                piece_counter += 1
                if piece_counter == 3:
                    found_mark = (x - 2, y, this_piece, "hztl")
                    all_found[found_mark] = piece_counter
                elif piece_counter > 3:
                    all_found[found_mark] = piece_counter
            else:
                piece_counter = 1
                found_mark = None
                piece_which = this_piece

        # breaking line
        piece_counter = 1
        found_mark = None
        piece_which = None

    # vertical
    piece_which = None
    piece_counter = None
    found_mark = None

    for x in rangesize:
        for y in rangesize:
            this_piece = board.get_piece(x, y)
            if this_piece == piece_which:
                piece_counter += 1
                if piece_counter == 3:
                    found_mark = (x, y - 2, this_piece, "vtcl")
                    all_found[found_mark] = piece_counter
                elif piece_counter > 3:
                    all_found[found_mark] = piece_counter
            else:
                piece_counter = 1
                found_mark = None
                piece_which = this_piece

        # breaking line
        piece_counter = 1
        found_mark = None
        piece_which = None

    # process!
    processed = []
    for (x, y, piece, piece_type), length in all_found.items():
        result = []
        if piece_type == "hztl":
            for i in range(length):
                result.append((x + i, y, piece))
        else:
            for i in range(length):
                result.append((x, y + i, piece))
        processed.append(result)

    while True:
        matching = _cross(processed)
        if not matching:
            break

        match_no = []
        match_yes = []
        for p in processed:
            if matching in p:
                match_yes.append(p)
            else:
                match_no.append(p)

        processed = match_no
        flattened = set()
        for m in match_yes:
            for x in m:
                flattened.add(x)
        processed.append(flattened)

    final = []
    for proc in processed:
        final.append(tuple((x, y) for x, y, __ in list(proc)))
    return final


def _get_sane_piece(board, pos_x, pos_y):
    """Return a piece for pos x, y different from roundings."""
    rounding = set()
    if pos_x > 0:
        rounding.add(board[pos_y][pos_x - 1])
    if pos_x < 7:
        rounding.add(board[pos_y][pos_x + 1])

    if pos_y > 0:
        rounding.add(board[pos_y - 1][pos_x])
    if pos_y < 7:
        rounding.add(board[pos_y + 1][pos_x])

    all_sane = rounding ^ set(range(7))
    return random.choice(list(all_sane))


def generate_nonmatch_board():
    """Return a board with no match."""
    def get_board():
        board = []
        for __ in range(8):
            lin = [random.randint(0, 6) for __ in range(8)]
            board.append(lin)
        return board

    board = get_board()

    while True:
        fb = FakeBoard(board)
        res = detect(fb)
        if res == []:
            # excelent! no match!
            if future_guess(fb):
                # yet better, we have a chance!
                return board

        if res:
            # need to alter sligthly the random one to eliminate matches
            pos_x, pos_y = res[0][0]
            pos_y = 7 - pos_y  # remember that the board is upside down
            sane = _get_sane_piece(board, pos_x, pos_y)
            board[pos_y][pos_x] = sane
        else:
            # no match, but also no chances... get a new board
            board = get_board()


def get_around(slot):
    """Return the slots all around the given one.

    Normally they're eight, unless the given slot is next to
    a wall, or in a corner.
    """
    px, py = slot
    for dx in (-1, 0, 1):
        for dy in (-1, 0, 1):
            if dx or dy:   # skip both in 0
                nx = px + dx
                ny = py + dy
                if 0 <= nx <= 7 and 0 <= ny <= 7:
                    yield nx, ny


def get_center(match, hints=None):
    """Guess the center in a match."""
    len_m = len(match)

    # the matches of three or four elements
    if len_m == 3:
        return match[1]

    if len_m == 4:
        m1, m2 = match[1], match[2]
        if hints is not None:
            for h in hints:
                if h == m1 or h == m2:
                    return h
        return m1

    # five elements and linear: just in the middle
    if len_m == 5 and \
       len(set(x[0] for x in match)) == 1 or \
       len(set(x[1] for x in match)) == 1:
        return match[2]

    # 5, 6, or 7 elements, can't be linear
    if 5 <= len_m <= 7:
        # it's an L, need to see the point where it turns
        xxx = [z[0] for z in match]
        for z in set(xxx):
            if xxx.count(z) >= 3:
                corner_x = z
                break
        else:
            raise ValueError("We had an L without good equals in X: %s", match)

        yyy = [z[1] for z in match]
        for z in set(yyy):
            if yyy.count(z) >= 3:
                corner_y = z
                break
        else:
            raise ValueError("We had an L without good equals in Y: %s", match)

        return (corner_x, corner_y)

    raise ValueError("Had a match with incorrect length: %s", match)


def generate_custom(board):
    """Return a board with specified layout."""
    # verify that all numbers are 0-6 (normal pieces),
    # 7-12 (explosives) or 13 (magic ball)
    for lin in board:
        for piece in lin:
            if not (0 <= piece <= 13):
                raise ValueError("Custom board has bad piece number: %r"
                                 % piece)

    # verify it doesn't has a match
    fb = FakeBoard(board)
    res = detect(fb)
    if res != []:
        raise ValueError("Custom board has matches: %s" % res)

    # all ok
    return board
