# This code is part of the 'enjuewemela' game
# License: GPLv3
# Main author: Facundo Batista
# Code, bug tracker, etc:
#   https://launchpad.net/enjuewemela/
#
"""Effects to remove pieces from the board."""

from __future__ import division

import math

from cocos.particle_systems import Explosion
from cocos.particle import ParticleSystem, Color

RAY_LIFE = 2


class _Ray(ParticleSystem):
    """A particle ray."""

    duration = RAY_LIFE

    #: The angle (direction) of the particles measured in degrees
    angle = 0.0
    #: Angle variance measured in degrees;
    angle_var = 1.0

    #: The speed the particles will have.
    speed = 200.0
    #: The speed variance
    speed_var = 0.0

    #: Size of the particles
    size = 13.0
    #: Size variance
    size_var = 20.0

    #: How many seconds will the particle live
    life = RAY_LIFE
    #: Life variance
    life_var = 0

    start_color = Color(1, 1, 1, 1.0)
    start_color_var = Color(0.5, 0.5, 0.5, 0.0)
    end_color = Color(0.5, 0.5, 0.5, 1.0)
    end_color_var = Color(0.5, 0.5, 0.5, 0.0)

    #: Maximum particles
    total_particles = 100

    emission_rate = 40


class Disappear(object):
    """Just disappear the piece immediately."""

    def __init__(self, pieces, from_user):
        self.pieces = list(pieces)
        self.from_user = from_user

    def remove(self, layer):
        """Remove the piece(s) from the layer."""
        return (self.pieces, 0, [])

    def score_before(self):
        """Score before the elapsed time."""
        return 'disappear', self.pieces, self.from_user

    def score_after(self):
        """Score after the elapsed time."""

    def __repr__(self):
        return "<Disappear pieces %s>" % (self.pieces,)


class Explode(object):
    """Explode all the pieces immediately, and small delay."""

    def __init__(self, expl_pieces, disap_pieces, from_user):
        self.expl_pieces = list(expl_pieces)
        self.disap_pieces = list(disap_pieces)
        self.from_user = from_user

    def remove(self, layer):
        """Remove the piece(s) from the layer."""
        for piece in self.expl_pieces:
            # smaller particles
            p1 = Explosion()
            p1.auto_remove_on_finish = True
            p1.speed = 160
            p1.speed_var = 230
            p1.total_particles = 30
            p1.size = 15
            p1.size_var = 10

            # bigger particles
            p2 = Explosion()
            p2.auto_remove_on_finish = True
            p2.speed = 60
            p2.speed_var = 30
            p2.total_particles = 30
            p2.size = 25
            p2.size_var = 10

            p1.position = p2.position = piece.position
            layer.add(p1)
            layer.add(p2)
        return (self.expl_pieces + self.disap_pieces, .5, [])

    def score_before(self):
        """Score before the elapsed time."""
        return 'explosion', self.expl_pieces, self.from_user

    def score_after(self):
        """Score after the elapsed time."""

    def __repr__(self):
        return "<Explode pieces %s  disapp %s>" % (self.expl_pieces,)


class Magic(object):
    def __init__(self, magic, similar_pieces):
        self.similar_pieces = similar_pieces
        self.magic = magic

    def remove(self, layer):
        """Remove the piece(s) from the layer."""
        center = self.magic.position

        destinations = [p.position for p in self.similar_pieces]
        for dest in destinations:
            delta_x = dest[0] - center[0]
            delta_y = dest[1] - center[1]
            dist = math.sqrt(delta_x ** 2 + delta_y ** 2)
            angrad = math.atan2(delta_y, delta_x)
            ang = angrad * 180 / math.pi

            p = _Ray()
            speed = dist / RAY_LIFE
            p.speed = speed
            p.position = center
            p.angle = ang
            p.destiny = dest
            layer.add(p)

        return ([], 2 * RAY_LIFE, [self.magic] + self.similar_pieces)

    def score_before(self):
        """Score before the elapsed time."""

    def score_after(self):
        """Score after the elapsed time."""
        return 'magic', [self.magic] + self.similar_pieces, False

    def __repr__(self):
        return "<Magic! node: %r  similars: %s>" % (self.magic,
                                                    self.similar_pieces)
