# This code is part of the 'enjuewemela' game
# License: GPLv3
# Main author: Facundo Batista
# Code, bug tracker, etc:
#   https://launchpad.net/enjuewemela/
#
"""Module to keep and show game stats."""

from __future__ import division

import pickle
import os

from operator import itemgetter

from xdg.BaseDirectory import xdg_data_home

from cocos import layer, sprite, actions
from cocos.director import director
from cocos.text import Label

import config
import utils

from pyglet import font
font.add_directory(os.path.join(config.BASEDIR, 'fonts'))

import logic_board
from gems import gems
from sound import sounds

from levels import (
    FRUKIS_LEVEL_LIMIT,
    INITIAL_FRUKIS,
    POINTS_PER_STYLE,
    get_level_factors,
)


class Stats(layer.Layer):
    """Keeps score and statistics."""

    def __init__(self, game_manager, game_style):
        super(Stats, self).__init__()

        self.score = 0
        self.destroyed = dict.fromkeys(range(14), 0)
        self.added = dict.fromkeys(range(7), 0)
        self.bombs = 0
        self.magics = 0
        self.specials = 0
        self.level = 0
        self.level_factors = get_level_factors(0)
        self.frukis = INITIAL_FRUKIS[game_style]
        self.gm = game_manager
        self.game_style = game_style

        # the fonts to use
        f_titles = utils.FontWriter(self, "Sansation")
        f_fixed = utils.FontWriter(self, "SV Basic Manual")

        # some info for the initial drawing
        win_w, win_h = director.get_window_size()
        side_w = (win_w - config.ARISTA * 8) // 2
        base_w, base_h = 100, 480

        #
        # ------ left side ---------
        #
        # draw a white area centered in height and in the left border
        left_border = (side_w - base_w) // 2
        lower_y = (win_h - base_h) // 2
        b = utils.Base(170, left_border, lower_y, base_w, base_h)
        self.add(b)

        middle = left_border + base_w // 2
        y = lower_y + base_h - 20

        # player
        f_titles("player", (middle, y), 9, "center",
                 color=(100, 127, 100, 255))
        y -= 20
        self.username = game_manager.config['username']
        f_titles(self.username, (middle, y), 15, "center",
                 color=(0, 77, 105, 255))

        # level
        y -= 40
        f_titles(_("level"), (middle, y), 9, "center",
                 color=(100, 127, 100, 255))
        y -= 20
        self.level_label = f_fixed("0", (middle, y), 20, "center",
                                      color=(0, 0, 255, 255))

        # score
        y -= 40
        f_titles(_("score"), (middle, y), 9, "center",
                 color=(100, 127, 100, 255))
        y -= 20
        self.score_label = f_fixed("0", (middle, y), 20, "center",
                                      color=(127, 0, 127, 255))

        # get all pieces for level advance
        self.advance_pieces = gems.get_all()
        smiley = sprite.Sprite("smiley.png")
        self.advance_pieces.append(smiley)

        # precalculate the max scale for all pieces
        piece_max_size = 40
        max_len = max(max(p.height, p.width) for p in self.advance_pieces)
        self.final_advance_scale = piece_max_size / max_len

        # draw them according to needed size
        separation = (base_w - piece_max_size * 2) // 3
        y -= 50
        for i in range(4):
            y -= 50

            # left
            p = self.advance_pieces[i * 2]
            p.position = (left_border + separation + piece_max_size // 2, y)
            p.scale = 0
            self.add(p)

            # right
            p = self.advance_pieces[i * 2 + 1]
            p.position = (left_border + separation * 2 +
                          int(piece_max_size * 1.5), y)
            p.scale = 0
            self.add(p)

        # game time
        self.game_time_label = f_fixed("0:00", (middle, lower_y + 10), 14,
                                          "center", color=(250, 50, 50, 255))

        #
        # ------ right side ---------
        #
        # draw a white area centered in height and in the right border
        left_border = win_w - (side_w - base_w) // 2 - base_w
        self.right_border = left_border + base_w
        lower_y = (win_h - base_h) // 2
        b = utils.Base(170, left_border, lower_y, base_w, base_h)
        self.add(b)

        stats_heights = []

        # hits counter
        y = lower_y + base_h - 15
        f_titles(_("Hits:"), (left_border + 5, y), 12, "left")
        for t in _("Pieces: Bombs: Magics: Specials:").split():
            y -= 17
            stats_heights.append(y)
            f_titles(t, (left_border + 10, y), 10, "left")

        # added pieces
        y -= 35
        f_titles(_("Appeared:"), (left_border + 5, y), 12, "left")
        all_gems = gems.get_all(20)
        for i, gem in enumerate(all_gems):
            y -= 22
            stats_heights.append(y)
            gem.position = (left_border + 35, y)
            self.add(gem)

        # destroyed pieces
        y -= 35
        f_titles(_("Destroyed:"), (left_border + 5, y), 12, "left")
        all_gems = gems.get_all(20)
        for i, gem in enumerate(all_gems):
            y -= 22
            stats_heights.append(y)
            gem.position = (left_border + 35, y)
            self.add(gem)

        # initial values, keeping the labels to update later
        self.stats_texts = []
        for y in stats_heights:
            l = f_fixed("0", (self.right_border - 15, y), 12, "right")
            self.stats_texts.append(l)

        self._adjust_advance_feedback()
        self.schedule_interval(self.update_timings, .5)

    def redraw_stats(self):
        """Redraw the statistic values."""
        # these are the values to show
        to_show = [sum(self.destroyed.values()), self.bombs, self.magics,
                                                                self.specials]
        to_show.extend(self.added[x] for x in range(7))
        to_show.extend(self.destroyed[x] for x in range(7))

        # show them!
        for lab, val in zip(self.stats_texts, to_show):
            lab.element.text = str(val)

    def do_score(self, match_type, pieces, from_user):
        """Update stats for all match types."""
        positions = [p.position for p in pieces]
        if match_type == 'magic':
            self._do_score_magic(positions)
        else:
            if match_type == 'explosion':
                explosive = [p for p in pieces if p.is_explosive][0]
                center = explosive.position
            else:
                center = logic_board.get_center(positions)
            self._do_score_normal(positions, from_user, match_type, center)

    def _do_score_magic(self, positions):
        """Update stats when there is magic action."""
        base = len(positions) ** 2
        points = int(round(base * self.level_factors.lmp))
        points *= POINTS_PER_STYLE[self.game_style]
        each = points // len(positions)

        # draw all the bubles and adjust the score
        for position in positions:
            self.score_bubble(position, each)

        self._do_score_adjust(points)

    def _do_score_normal(self, positions, from_user, match_type, center):
        """Update stats when there is a gem match."""
        points = 0
        points += len(positions) * self.level_factors.lmp
        points *= POINTS_PER_STYLE[self.game_style]

        # adjust score accordingly to the length of match
        points *= len(positions)

        # if the match is because falling pieces, it'x 2x
        if not from_user:
            points *= 2

        # some special score
        if match_type == 'explode':
            points += 1.5

        # draw the bubble and adjust the score
        self.score_bubble(center, int(round(points)))
        self._do_score_adjust(points)

    def _do_score_adjust(self, points):
        """Adjust the score for the given points."""
        # adjust score
        self.score += points
        self.score_label.element.text = str(int(self.score))
        self.frukis += points * self.level_factors.fpp
        self.redraw_stats()

    def score_bubble(self, position, points):
        """Shows a score that floats and fades away."""
        lab = Label(str(points), position, font_name="Komika Krak",
                    font_size=30, color=(0, 0, 55, 255), anchor_x="center")
        self.add(lab)

        def remove():
            """Removes the label."""
            self.remove(lab)

        lab.do(actions.FadeOut(config.BUBBLE_LIFE))
        lab.do(actions.MoveBy((0, 70), config.BUBBLE_LIFE))
        self.call_later(config.BUBBLE_LIFE, remove)

    def destroy_piece(self, piece):
        """Counts destroyed pieces."""
        self.destroyed[piece.pos_id] += 1

    def add_piece(self, piece):
        """Counts added pieces."""
        pos = piece.pos_id
        if pos < 13:
            self.added[pos] += 1

    def __str__(self):
        dt = self.gm.game_time
        return "Stats(time %d, score %d, added: %s, destroyed: %s)" % (
                                    dt, self.score, self.added, self.destroyed)

    def update_timings(self, dt):
        """Update game timings."""
        # time label
        time_text = "%d:%02d" % divmod(self.gm.game_time, 60)
        self.game_time_label.element.text = time_text

        # adjust frukis regarding time
        x = self.level_factors.fps * dt
        self.frukis -= x
        if self.game_style in ('rush', 'crazy') and self.frukis < 0:
            self.unschedule(self.update_timings)
            self.gm.pm.game_over()

        # need to check this here and not where we added frukis, because time
        # may have lowered it below the limit
        if self.frukis >= FRUKIS_LEVEL_LIMIT:
            sounds.play("new_level")
            self.frukis = INITIAL_FRUKIS[self.game_style]
            self.level += 1
            self.level_factors = get_level_factors(self.level)
            self.level_label.element.text = str(self.level)

        self._adjust_advance_feedback()

    def _adjust_advance_feedback(self):
        """Show the feedback pieces and smily in differentsizes."""
        fulls, graduated = divmod(self.frukis * 8, FRUKIS_LEVEL_LIMIT)
        for i, p in enumerate(self.advance_pieces):
            if i < fulls:
                p.scale = self.final_advance_scale
            elif i == fulls:
                p.scale = (self.final_advance_scale * graduated
                                                        / FRUKIS_LEVEL_LIMIT)
            else:
                p.scale = 0

    def save_score(self):
        """Save the score for the player."""
        fname = os.path.join(xdg_data_home, 'enjuewemela.hs')
        if os.path.exists(fname):
            try:
                with open(fname, 'rb') as fh:
                    all_hs = pickle.load(fh)
            except Exception, e:
                print "WARNING: problems with the high score filename:", e
                return
        else:
            print "WARNING: creating high score file from scratch"
            all_hs = {}

        hs = all_hs.setdefault(self.game_style, [])
        hs.append((self.username, int(self.score)))
        hs.sort(key=itemgetter(1), reverse=True)
        hs[50:] = []    # not unbound

        with open(fname, 'wb') as fh:
            pickle.dump(all_hs, fh)

    def call_later(self, dt, func, *args, **kwargs):
        """Handy method to call later a function."""
        def f(dt, args, kwargs):
            """Unschedule the scheduled function."""
            self.unschedule(f)
            func(*args, **kwargs)
        self.schedule_interval(f, dt, args, kwargs)
