# This code is part of the 'enjuewemela' game
# License: GPLv3
# Main author: Facundo Batista
# Code, bug tracker, etc:
#   https://launchpad.net/enjuewemela/
#
"""Tests for the detector."""

import unittest

from enjuewemela.logic_board import detect, generate_nonmatch_board, FakeBoard

class TestHorizontal(unittest.TestCase):

    def test_empty(self):
        '''No three in a row.'''
        b = ("wxyz",
             "zrst",
             "wxyz",
             "abcd",
            )

        res = detect(FakeBoard(b))
        self.assertEqual(res, [])

    def test_crossline(self):
        '''Three only formed through boundaries (not allowed).'''
        b = ("wdyz",
             "zr..",
             ".pyz",
             "bpcd",
            )

        res = detect(FakeBoard(b))
        self.assertEqual(res, [])

    def test_simple(self):
        '''Simple match.'''
        b = ("gxyz",
             "h...",
             "jxyz",
             "abcd",
            )

        res = detect(FakeBoard(b))
        self.assertEqual(len(res), 1)
        self.assertEqual(res[0], ((1,2), (2,2), (3,2)))

    def test_big_scenario(self):
        '''Three in a row, twice, with a big scenario.'''
        b = ("fxyzhtxg",
             "drs...jn",
             "hxyvwdfb",
             "abcyqwef",
             "dr...ojn",
             "abcdqwef",
             "hxyzwdfb",
             "abcdqwef",
            )

        res = sorted(detect(FakeBoard(b)))
        self.assertEqual(len(res), 2)
        self.assertEqual(res[0], ((2,3), (3,3), (4,3)))
        self.assertEqual(res[1], ((3,6), (4,6), (5,6)))

    def test_double(self):
        b = ("h...",
             "...r",
             "kxyd",
             "abcs",
            )

        res = sorted(detect(FakeBoard(b)))
        self.assertEqual(len(res), 2)
        self.assertEqual(res[0], ((0,2), (1,2), (2,2)))
        self.assertEqual(res[1], ((1,3), (2,3), (3,3)))

    def test_double_different(self):
        b = ("...z",
             "drsd",
             "axyd",
             "+++s",
            )

        res = sorted(detect(FakeBoard(b)))
        self.assertEqual(len(res), 2)
        self.assertEqual(res[0], ((0,0), (1,0), (2,0)))
        self.assertEqual(res[1], ((0,3), (1,3), (2,3)))

    def test_four(self):
        '''Four in a row.'''
        b = ("xdyz",
             "dfst",
             "....",
             "ahcd",
            )

        res = detect(FakeBoard(b))
        self.assertEqual(len(res), 1)
        self.assertEqual(res[0], ((0,1), (1,1), (2,1), (3, 1)))

    def test_five(self):
        '''Five in a row.'''
        b = ("hxykjz",
             "grfdst",
             "g.....",
             "wxygfz",
             "grfhst",
             "abgjcd",
            )

        res = detect(FakeBoard(b))
        self.assertEqual(len(res), 1)
        self.assertEqual(res[0], ((1,3), (2,3), (3,3), (4,3), (5,3)))


class TestVertical(unittest.TestCase):

    def test_empty(self):
        '''No three in a row.'''
        b = ("wxyz",
             "zrst",
             "wxyz",
             "abcd",
            )

        res = detect(FakeBoard(b))
        self.assertEqual(res, [])

    def test_crossline(self):
        '''Three only formed through boundaries (not allowed).'''
        b = ("w.yz",
             "zrst",
             ".pyz",
             ".pcd",
            )

        res = detect(FakeBoard(b))
        self.assertEqual(res, [])

    def test_simple(self):
        '''Simple match.'''
        b = (".xyz",
             ".rst",
             ".xyz",
             "abcd",
            )

        res = detect(FakeBoard(b))
        self.assertEqual(len(res), 1)
        self.assertEqual(res[0], ((0,1), (0,2), (0,3)))

    def test_big_scenario(self):
        '''Three in a row, with a big scenario.'''
        b = ("fxyzhtxg",
             "drstlojn",
             "hxy.wdfb",
             "abc.qwef",
             "drs.lojn",
             "abcdqwef",
             "hxyzwdfb",
             "abcdqwef",
            )

        res = detect(FakeBoard(b))
        self.assertEqual(len(res), 1)
        self.assertEqual(res[0], ((3,3), (3,4), (3,5)))

    def test_double(self):
        b = (".xyz",
             ".rs.",
             ".xy.",
             "abc.",
            )

        res = sorted(detect(FakeBoard(b)))
        self.assertEqual(len(res), 2)
        self.assertEqual(res[0], ((0,1), (0,2), (0,3)))
        self.assertEqual(res[1], ((3,0), (3,1), (3,2)))

    def test_double_different(self):
        b = (".xyz",
             ".rs+",
             ".xy+",
             "abc+",
            )

        res = sorted(detect(FakeBoard(b)))
        self.assertEqual(len(res), 2)
        self.assertEqual(res[0], ((0,1), (0,2), (0,3)))
        self.assertEqual(res[1], ((3,0), (3,1), (3,2)))

    def test_four(self):
        '''Four in a row.'''
        b = ("x.yz",
             "d.st",
             "g.yz",
             "a.cd",
            )

        res = detect(FakeBoard(b))
        self.assertEqual(len(res), 1)
        self.assertEqual(res[0], ((1,0), (1,1), (1,2), (1, 3)))

    def test_five(self):
        '''Five in a row.'''
        b = ("hxykjz",
             "grf.st",
             "grf.st",
             "wxy.fz",
             "grf.st",
             "abg.cd",
            )

        res = detect(FakeBoard(b))
        self.assertEqual(len(res), 1)
        self.assertEqual(res[0], ((3,0), (3,1), (3,2), (3,3), (3,4)))


class TestComplex(unittest.TestCase):

    def test_L_ul(self):
        '''Upper left L.'''
        b = ("wxyz",
             "...x",
             ".xyz",
             ".bcd",
            )

        res = detect(FakeBoard(b))
        self.assertEqual(len(res), 1)
        self.assertEqual(sorted(res[0]), [(0,0), (0,1), (0,2), (1,2), (2,2)])

    def test_L_lr(self):
        '''Lower right L.'''
        b = ("wxyz",
             "av.x",
             "vx.z",
             "...d",
            )

        res = detect(FakeBoard(b))
        self.assertEqual(len(res), 1)
        self.assertEqual(sorted(res[0]), [(0,0), (1,0), (2,0), (2,1), (2,2)])

    def test_L_ul_simple(self):
        '''Upper left L, plus a simple one.'''
        b = ("wxyzy",
             "...xt",
             ".xy+r",
             ".bc+e",
             "ldi+n",
            )

        res = sorted(detect(FakeBoard(b)))
        self.assertEqual(len(res), 2)
        self.assertEqual(sorted(res[0]), [(0,1), (0,2), (0,3), (1,3), (2,3)])
        self.assertEqual(res[1], ((3,0), (3,1), (3,2)))

    def test_T_simple(self):
        '''Simple T.'''
        b = ("wxyzy",
             "asdxt",
             "s...d",
             "gb.fe",
             "ld.fn",
            )

        res = sorted(detect(FakeBoard(b)))
        self.assertEqual(len(res), 1)
        self.assertEqual(sorted(res[0]), [(1,2), (2,0), (2,1), (2,2), (3,2)])

    def test_T_plus1(self):
        '''T with extra roof 1.'''
        b = ("wxyzy",
             "a.dxt",
             "s.dsd",
             "g...e",
             "l.qfn",
            )

        res = sorted(detect(FakeBoard(b)))
        self.assertEqual(len(res), 1)
        self.assertEqual(sorted(res[0]),
                         [(1,0), (1,1), (1,2), (1,3), (2,1), (3,1)])

    def test_T_plus2(self):
        '''T with extra roof 2.'''
        b = ("wxyzyo",
             "afdxti",
             "ssd.du",
             "gdf.ev",
             "l....t",
             "llsdmt",
            )

        res = sorted(detect(FakeBoard(b)))
        self.assertEqual(len(res), 1)
        self.assertEqual(sorted(res[0]),
                         [(1,1), (2,1), (3,1), (3,2), (3,3), (4,1)])

    def test_super_cross(self):
        '''T with extra legs.'''
        b = ("wxyzyo",
             "af.dsi",
             "ss.fdu",
             ".....v",
             "lhddat",
             "llsdmt",
            )

        res = sorted(detect(FakeBoard(b)))
        self.assertEqual(len(res), 1)
        self.assertEqual(sorted(res[0]),
                         [(0,2), (1,2), (2,2), (2,3), (2,4), (3,2), (4,2)])

    def test_real_life_1(self):
        '''Example found when testing.'''
        b = [[1, 4, 1, 5, 1, 2, 4, 6],
             [6, 4, 6, 6, 6, 6, 5, 5],
             [1, 1, 5, 6, 5, 1, 4, 6],
             [5, 5, 3, 6, 5, 2, 5, 4],
             [5, 6, 6, 5, 6, 0, 0, 5],
             [3, 0, 6, 1, 4, 1, 2, 4],
             [4, 0, 1, 6, 0, 4, 6, 1],
             [4, 1, 4, 2, 5, 5, 5, 4]
            ]

        res = sorted(detect(FakeBoard(b)))
        self.assertEqual(len(res), 2)
        self.assertEqual(sorted(res[0]),
                                [(2, 6), (3, 4), (3,5), (3,6), (4,6), (5,6)])
        self.assertEqual(sorted(res[1]), [(4,0), (5,0), (6,0)])


class TestNonMatch(unittest.TestCase):
    """Test the nonmatch board generation."""

    def test_simple(self):
        """Test to get random boards several times."""
        for i in range(1000):
            b = generate_nonmatch_board()
            res = detect(FakeBoard(b))
            self.assertEqual(res, [])
